Imports System
Imports System.Data
Imports System.Data.Sql
Imports System.Data.SqlTypes
Imports Microsoft.SqlServer.Server
Imports Microsoft.AnalysisServices.AdomdClient


Partial Public Class StoredProcedures
    <Microsoft.SqlServer.Server.SqlProcedure()> _
Public Shared Sub CreateSessionModel(ByVal Year As Integer, ByVal numberMonths As Integer)

        Dim query As String
        Dim connection As AdomdConnection = New AdomdConnection("provider=msolap;data source=localhost;initial catalog=Adventure Works DW")
        Dim command As New AdomdCommand()
        Dim adapter As New AdomdDataAdapter()

        Try

            connection.Open()
            ' Drop mining structure if exists from a previous session
            Try
                command.CommandText = "DROP MINING STRUCTURE SessionForecast_Structure"
                command.Connection = connection
                command.ExecuteNonQuery()
            Catch ex As Exception
                ' No big deal if we fail; structure simply doesn't exists
            End Try

            ' Create a session model
            query = "CREATE SESSION MINING MODEL SessionForecast ( " _
                    & "TimeIndex LONG KEY TIME, " _
                    & "ModelRegion TEXT KEY," _
                    & "Amount DOUBLE CONTINUOUS PREDICT" _
                    & ") Using Microsoft_Time_Series WITH DRILLTHROUGH"

            command.CommandText = query
            command.ExecuteNonQuery()

            Dim startMonth As Integer = Int32.Parse(Year.ToString() & "01")
            Dim endMonth As Integer = Int32.Parse(Year.ToString() & "12")

            ' Train the model
            query = "INSERT INTO SessionForecast " _
                    & "(TimeIndex, ModelRegion, Amount) " _
                    & "OPENQUERY([Adventure Works DW], " _
                    & "'Select TimeIndex, ModelRegion, Amount " _
                    & "FROM [vTimeSeries] " _
                    & "WHERE (TimeIndex >= " & startMonth & " And TimeIndex <= " & endMonth & ")" _
                    & "ORDER BY TimeIndex')"


            command.CommandText = query
            command.ExecuteNonQuery()

            Dim predictions As New DataSet
            ' Get predicted results
            query = "SELECT ModelRegion, " _
                    & "(SELECT $TIME, [Amount], PredictVariance([Amount]) " _
                    & "FROM PredictTimeSeries([Amount], @NumberMonths)) " _
                    & "FROM [SessionForecast]"

            command.CommandText = query
            command.Parameters.Add("NumberMonths", numberMonths)
            adapter.SelectCommand = command
            ' Fill the dataset
            adapter.Fill(predictions)

            ' Get the cases
            query = "SELECT ModelRegion, TimeIndex, Amount  FROM [SessionForecast].CASES"
            command.CommandText = query
            adapter.SelectCommand = command
            Dim cases As New DataTable
            adapter.Fill(cases)

            ' Merge cases and predictions            
            Dim results As DataTable = PopulatePredictedColumns(predictions.Tables(0))
            results.Merge(cases)

            Dim record As SqlDataRecord = TableRowToSqlDataRecord(results)
            SqlContext.Pipe.SendResultsStart(record)

            For Each row As DataRow In results.Rows
                record.SetValues(row.ItemArray)
                SqlContext.Pipe.SendResultsRow(record)
            Next

            SqlContext.Pipe.SendResultsEnd()

        Catch ex As Exception
            Throw ex
        Finally
            ' Close the connection and destroy session
            ' to destroy the session mining model
            connection.Close(True)
        End Try
    End Sub

    Private Shared Function PopulatePredictedColumns(ByVal predictions As DataTable) As DataTable

        Dim results As New DataTable()
        results.Columns.Add("ModelRegion", System.Type.GetType("System.String"))
        results.Columns.Add("TimeIndex", System.Type.GetType("System.Int64"))
        results.Columns.Add("Amount", System.Type.GetType("System.Double"))

        Dim nestedRows As DataRow()
        Dim relation As DataRelation = predictions.DataSet.Relations(0)

        For Each caseRow As DataRow In predictions.Rows
            nestedRows = caseRow.GetChildRows(relation)
            For Each nestedRow As DataRow In nestedRows
                results.Rows.Add(caseRow(0), nestedRow(0), nestedRow(1))
            Next
        Next

        Return results
    End Function

    Private Shared Function TableRowToSqlDataRecord(ByVal table As DataTable) As SqlDataRecord

        Dim meta(table.Columns.Count - 1) As SqlMetaData

        'Dim schemaTable As DataTable = table.CreateDataReader().GetSchemaTable()
        '
        Dim collationString As New SqlString(String.Empty)

        Dim i As Integer

        For i = 0 To meta.Length - 1

            ' Dim columnSchema As DataRow = schemaTable.Rows(i)
            Dim column As DataColumn = table.Columns(i)

            Select Case column.DataType.ToString().ToLower()
                Case "system.int32"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.Int)
                Case "system.int64"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.BigInt)
                Case "system.string"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.NVarChar, column.MaxLength)
                Case "system.decimal"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.Decimal, 19, 4)
                Case "system.double"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.Float)
                Case "system.guid"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.UniqueIdentifier)
                Case "system.datetime"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.DateTime)
                Case "system.double"
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.Decimal, 19, 4)
                Case Else
                    meta(i) = New SqlMetaData(column.Caption, SqlDbType.NVarChar, column.MaxLength)
            End Select
        Next

        Dim rec As New SqlDataRecord(meta)
        Return rec
    End Function


End Class
