Imports System.IO
Imports System.Xml
Imports System.Data
Imports Microsoft.AnalysisServices.AdomdClient

Public Class Main

    Private m_sessionID As String


    ''' <summary>
    ''' Connects to the specified SSAS database
    ''' </summary>
    ''' <remarks></remarks>
    Private ReadOnly Property Connect() As AdomdConnection

        Get
            Dim connection As AdomdConnection

            Try
Connect:
                ' Try to reuse the existing session
                connection = New AdomdConnection(Me.txtConnectionString.Text)
                connection.SessionID = m_sessionID
                connection.Open()
                m_sessionID = connection.SessionID      ' Cache the session ID
                Return connection

            Catch ex As AdomdConnectionException
                If (ex.InnerException Is Nothing) = False Then
                    ' Session not found error
                    If ex.InnerException.Message.Contains(m_sessionID) Then
                        m_sessionID = Nothing
                        ' Try again with no session
                        GoTo Connect
                        grpQuery.Enabled = True
                    Else
                        Throw ex
                    End If
                End If

            Catch ex As Exception
                Throw ex
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End Get

    End Property



            ''' <summary>
            ''' Executes the query and gets a data reader
            ''' </summary>
            ''' <remarks></remarks>
    Private Sub DoDataReader()

        Dim connection As AdomdConnection = Me.Connect
        Dim command As New AdomdCommand(txtQuery.Text, connection)
        Dim reader As AdomdDataReader
        Dim lvItem As ListViewItem
        Dim value As String

        ' Clear the list control
        lsvResults.Items.Clear()
        lsvResults.Columns.Clear()

        ' Show the control
        lsvResults.Visible = True
        dgvResults.Visible = False
        wbrResults.Visible = False

        Try
            connection = Connect()
            ' Execute the query
            reader = command.ExecuteReader(CommandBehavior.CloseConnection)

            ' Get the schema info to configure the list view
            Dim schema As DataTable = reader.GetSchemaTable()

            For Each row As DataRow In schema.Rows
                lsvResults.Columns.Add(row("ColumnName"), 200, HorizontalAlignment.Left)
            Next

            ' Get data
            Do While reader.Read()

                lvItem = New ListViewItem()
                For i As Integer = 0 To reader.FieldCount - 1
                    value = IIf(reader.IsDBNull(i), String.Empty, reader(i).ToString())
                    If i = 0 Then
                        lvItem.Text = value
                    Else
                        lvItem.SubItems.Add(value)
                    End If
                Next
                lsvResults.Items.Add(lvItem)
            Loop
        Catch ex As Exception
            Throw ex
        Finally
            ' Close the DataReader
            If Not (reader Is Nothing) Then
                reader.Close()
            End If
        End Try

    End Sub

    ''' <summary>
    ''' Executes the query and gets a dataset
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DoDataSet()
        Dim connection As AdomdConnection = Me.Connect
        Dim command As New AdomdCommand(txtQuery.Text, connection)
        Dim adapter As New AdomdDataAdapter(command)
        Dim dataset As New DataSet

        ' Show DataGrid
        lsvResults.Visible = False
        dgvResults.Visible = True
        wbrResults.Visible = False

        ' Fill the dataset
        adapter.Fill(dataset)

        ' Bind to grid
        dgvResults.DataSource = dataset.Tables(0)
        connection.Close(False)


    End Sub
    ''' <summary>
    ''' Executes the query and gets a CellSet
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DoCellSet()

        Dim connection As AdomdConnection = Me.Connect
        Dim command As New AdomdCommand(txtQuery.Text, connection)
        Dim cellset As CellSet
        Dim lvItem As ListViewItem

        ' Clear the list control
        lsvResults.Items.Clear()
        lsvResults.Columns.Clear()

        ' Show the control
        lsvResults.Visible = True
        dgvResults.Visible = False
        wbrResults.Visible = False

        ' Execute the query
        cellset = command.ExecuteCellSet()


        lsvResults.Columns.Add("Cells", 200, HorizontalAlignment.Left)

        ' Display first two axes
        For i As Integer = 0 To cellset.Axes(0).Positions.Count - 1
            For j As Integer = 0 To cellset.Axes(1).Set.Tuples.Count - 1
                lvItem = New ListViewItem(cellset.Axes(0).Set.Tuples(i).Members(0).Caption & _
                 ", " & cellset.Axes(1).Set.Tuples(j).Members(0).Caption & ": " & _
                 cellset.Cells(i, j).FormattedValue)
                lsvResults.Items.Add(lvItem)
            Next
        Next

        ' Load FilterAxis info into a StringBuilder object
        Dim stringBuilder As New System.Text.StringBuilder
        With cellset.FilterAxis

            For Each fposCurrent As Position In .Positions
                stringBuilder.Append("Position #" & fposCurrent.Ordinal & ":" & vbCrLf)
                For Each fposmemCurrent As Member In fposCurrent.Members
                    stringBuilder.Append(fposmemCurrent.Caption & vbCrLf)
                Next
            Next fposCurrent

        End With

        System.Diagnostics.Trace.WriteLine(stringBuilder.ToString())
        connection.Close(False)
    End Sub

    ''' <summary>
    ''' Executes the query and gets an XMLA cellset
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DoXMLA()

        Dim connection As AdomdConnection = Me.Connect
        Dim command As New AdomdCommand(txtQuery.Text, connection)
        Dim xmlReader As System.Xml.XmlReader
        Dim doc As New XmlDocument()
        Dim filePath As String

        Dim xdcResults As New Xml.XmlDocument

        ' Show the Web Browser control
        lsvResults.Visible = False
        dgvResults.Visible = False
        wbrResults.Visible = True

        filePath = Application.StartupPath + "\out.xml"

        Try
            ' Execute XML query
            xmlReader = command.ExecuteXmlReader()
            doc.Load(xmlReader)
            doc.Save(filePath)
            ' Display the results in the web browser
            wbrResults.Navigate(filePath)

        Finally
            xmlReader.Close()
            connection.Close(False)
        End Try

    End Sub

    '/ <summary>
    '/ Gets the user preferences from the isolated storage
    '/ </summary>
    Sub LoadUserSettings()
        Try
            Dim dom As New XmlDocument()
            Dim s As Stream = Util.GetSettingsStream(Me.Name, FileMode.Open, FileAccess.Read, FileShare.Read)
            dom.Load(s)
            txtConnectionString.Text = dom.SelectSingleNode("//connectionString").InnerText
            txtQuery.Text = dom.SelectSingleNode("//queryText").InnerText
        Catch ' no big deal if we fail - this is a convenience feature only
        End Try
    End Sub 'LoadUserSettings

    '/ <summary>
    '/ Saves the user preferences to the isolated storage
    '/ </summary>
    Sub SaveUserSettings()
        Try
            Dim s As Stream = Util.GetSettingsStream(Me.Name, FileMode.Create, FileAccess.ReadWrite, FileShare.None)
            Dim w As New XmlTextWriter(s, System.Text.Encoding.UTF8)
            w.WriteStartElement(Me.Name)
            w.WriteElementString("connectionString", txtConnectionString.Text)
            w.WriteElementString("queryText", txtQuery.Text)
            w.WriteEndElement()
            w.Close()
        Catch ' no big deal if we fail - this is a convenience feature only
        End Try
    End Sub 'SaveUserSettings
    Private Sub Main_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        LoadUserSettings()
    End Sub
    Private Sub Main_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        SaveUserSettings()
    End Sub


    Private Sub btnSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSend.Click

        Try
            Me.Cursor = Cursors.WaitCursor
            If radDataReader.Checked Then
                DoDataReader()
            ElseIf radDataSet.Checked Then
                DoDataSet()
            ElseIf radCellSet.Checked Then
                DoCellSet()
            ElseIf radXMLA.Checked Then
                DoXMLA()
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Return
        Finally
            Me.Cursor = Cursors.Default

        End Try

    End Sub
End Class
