Imports System
Imports System.IO
Imports System.Xml
Imports System.Data
Imports System.Configuration
Imports InteractiveReporter.AW ' For the MyReports Web service
Imports Microsoft.Office.Interop.Owc11
Imports AxMicrosoft.Office.Interop.Owc11
Imports Microsoft.AnalysisServices.AdomdClient

Public Class Main
#Region "my vars"
    Private m_reports As DataSet = Nothing
    Private m_dirty As Boolean = False
#End Region


    Private Sub ConfigureOWC()
        pivotTable.ConnectionString = My.Settings.SOS_OLAP
    End Sub 'ConfigureOWC
    ''' <summary>
    ''' A Dirty flag to tells us if a report has been created, changed, deleted
    ''' </summary>
    ''' <value></value>
    ''' <remarks></remarks>
    Private Property Dirty() As Boolean
        Get
            Return m_dirty
        End Get
        Set(ByVal value As Boolean)
            m_dirty = value
            btnSave.Enabled = Me.Dirty
        End Set
    End Property
    ''' <summary>
    ''' Creates a chart from the Pivot report
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadChart()
        Dim chart As ChChart
        Dim pview As PivotView = pivotTable.ActiveView
        ' Bind the Chart control to the PivotTable control if necessary
        chartSpace.DataSource = CType(pivotTable.GetOcx(), msdatasrc.DataSource)
        chartSpace.DisplayFieldList = True
        chartSpace.AllowUISelection = True

        'Get the chart or create one if needed
        If chartSpace.Charts.Count = 0 Then
            chart = chartSpace.Charts.Add(0)
        Else
            chart = chartSpace.Charts(0)
        End If
        'Set the title to the PivotTable title
        chart.HasTitle = True
        chart.Title.Caption = pview.TitleBar.Caption
    End Sub 'LoadChart

    ''' <summary>
    ''' Calls to the MyReports Web service to get the custom and standard reports
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadMyReports()
        Dim myReports As New MyReports()
        myReports.Url = My.Settings.MyReportsFacadeUrl
        myReports.Credentials = System.Net.CredentialCache.DefaultCredentials
        'cmbReports.SelectedIndex = -1;
        cmbReports.DataBindings.Clear()
        m_reports = myReports.GetReports()

        ' define a primary key for this table so we can search on it
        m_reports.Tables(0).PrimaryKey = New DataColumn() {m_reports.Tables(0).Columns(0)}
        cmbReports.DisplayMember = "Name"
        cmbReports.ValueMember = "ReportID"
        cmbReports.DataSource = m_reports.Tables(0)
        Me.Refresh()
    End Sub 'LoadMyReports
    ''' <summary>
    ''' Saves the report changes to a dataset
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SaveReport(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnNew.Click, btnUpdate.Click, btnDelete.Click

        Dim button As Button = CType(sender, Button)
        Select Case button.Name
            Case "btnNew"
                Dim reportName As String = Interaction.InputBox("Enter the report name", "New Report", Nothing, -1, -1)
                If reportName.Trim() <> String.Empty Then
                    ' add a new report
                    Dim newReport As DataRow = m_reports.Tables(0).NewRow()
                    Dim tempID As Integer = -(m_reports.Tables(0).Rows.Count + 1)
                    newReport("ReportID") = tempID
                    newReport("Name") = reportName
                    newReport("ReportDef") = pivotTable.XMLData
                    m_reports.Tables(0).Rows.Add(newReport)
                    cmbReports.SelectedValue = tempID
                    Dirty = True
                End If

            Case "btnUpdate"
                ' update an existing report
                Dim report As DataRow = m_reports.Tables(0).Rows.Find(cmbReports.SelectedValue)
                report("ReportDef") = pivotTable.XMLData
                Dirty = True

            Case "btnDelete"
                Dim report As DataRow = m_reports.Tables(0).Rows.Find(cmbReports.SelectedValue)
                report.Delete()
                Dirty = True
                LoadReport(CInt(cmbReports.SelectedValue))
        End Select
    End Sub 'SaveReport
    ''' <summary>
    ''' Binds the report definition to the PivotTable control
    ''' </summary>
    ''' <param name="reportID"></param>
    ''' <remarks></remarks>
    Private Sub LoadReport(ByVal reportID As Integer)
        Dim report As DataRow = m_reports.Tables(0).Rows.Find(reportID)
        pivotTable.XMLData = report("ReportDef").ToString()
        pivotTable.ActiveView.TitleBar.Caption = cmbReports.Text
        pivotTable.AutoFit = False
        pivotTable.Width = tabPivot.Width - 10
        pivotTable.Height = tabPivot.Height - 20
    End Sub 'LoadReport

    ''' <summary>
    ''' Configures the user interface to allow/disallow changes to a report based on the report type
    ''' </summary>
    ''' <param name="reportID"></param>
    ''' <remarks></remarks>
    Private Sub ConfigureMyReports(ByVal reportID As Integer)
        Dim report As DataRow = m_reports.Tables(0).Rows.Find(reportID)
        If report("Category").ToString().ToLower() = "standard" Then
            btnUpdate.Enabled = (btnDelete.Enabled = False)
        Else
            btnUpdate.Enabled = (btnDelete.Enabled = True)
        End If
    End Sub 'ConfigureMyReports
    ''' <summary>
    ''' Persist the report changes by calling down to the ReportService
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateMyReports()
        Dim changedCount As Integer = 0
        Dim selectedValue As Integer = CInt(cmbReports.SelectedValue)
        'int selectedValue = (int) cmbReports.SelectedValue;
        Dim changedReports As DataSet = m_reports.GetChanges()
        changedCount = changedReports.Tables(0).Rows.Count
        Dim myReports As New MyReports()
        myReports.Credentials = System.Net.CredentialCache.DefaultCredentials
        myReports.SaveReports(changedReports)

        MessageBox.Show((changedCount.ToString() + " report(s) updated successfully."), Me.Text)
        LoadMyReports()

        If selectedValue > 0 Then
            cmbReports.SelectedValue = selectedValue ' not a new row
        End If
    End Sub 'UpdateMyReports

    ''' <summary>
    ''' Determine whether a valid selection is made to display the drillthrough report
    ''' A valid selection has the Time dimension expanded to months</summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function IsSelectionValid() As Boolean
        Dim selection As String = pivotTable.SelectionType
        Dim aggregates As PivotAggregates = Nothing
        Dim aggregate As PivotAggregate = Nothing
        Dim valid As Boolean = False
        Dim uniqueName As String = Nothing

        If selection <> "PivotAggregates" Then
            Return False 'data cell is not selected
        End If
        aggregates = CType(pivotTable.Selection, PivotAggregates)
        aggregate = aggregates(0)

        If TypeOf aggregate.Value Is System.DBNull Then
            Return False 'no data
        End If
        ' seek the time dimension in columns and rows
        'uniqueName = aggregate.Cell.ColumnMember.UniqueName
        valid = Not (GetMonthMember(aggregate.Cell) Is Nothing)
        Return valid
    End Function 'IsSelectionValid
    ''' <summary>
    ''' Determines if the Date dimension is expanded down to the Month level
    ''' by checking both column and row axis
    ''' </summary>
    ''' <param name="cell"></param>
    ''' <returns>The Month member if exists; otherwise Nothing</returns>
    ''' <remarks></remarks>
    Private Function GetMonthMember(ByVal cell As PivotCell) As PivotField
        ' Try columns
        For Each field As PivotField In cell.ColumnMember.Axis.Fields
            If field.Type = PivotFieldTypeEnum.plTypeTimeMonths Then
                Return field
            End If
        Next
        ' Try rows
        For Each field As PivotField In cell.RowMember.Axis.Fields
            If field.Type = PivotFieldTypeEnum.plTypeTimeMonths Then
                Return field
            End If
        Next
        Return Nothing
    End Function

    ''' <summary>
    ''' Runs a Reporting Services report
    ''' </summary>
    ''' <param name="reportName"></param>
    ''' <remarks></remarks>
    Private Sub RunReport(ByVal reportName As String)
        Dim reportURL As String = Nothing

        ' Gets the month member
        Dim aggregates As PivotAggregates = CType(pivotTable.Selection, PivotAggregates)
        Dim month As PivotField = GetMonthMember(aggregates(0).Cell)

        If month Is Nothing Then
            Return
        End If

        If aggregates(0).Cell.ColumnMember.UniqueName.Contains("[Month]") = False Then
            MessageBox.Show("Please expand to the Date hieararchy to the Month level.", Me.Text)
            Exit Sub
        End If

        Try
            Dim reportViewer As New ReportViewer()
            reportViewer.ReportName = reportName
            reportViewer.ReportDate = GetDateReportParameter(aggregates(0).Cell.ColumnMember.UniqueName)
            reportViewer.Show()

        Catch ex As System.Exception
            Util.ShowErrorMessage(ex)
            Throw
        End Try

    End Sub 'RunReport


    ''' <summary>
    ''' Extracts the date from the Date dimension
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetDateReportParameter(ByVal memberName As String) As DateTime
        Dim aggregates As PivotAggregates = CType(pivotTable.Selection, PivotAggregates)
        Dim aggregate As PivotAggregate = aggregates(0)

        memberName = memberName.Substring(memberName.IndexOf("&"))
        Dim members As String() = memberName.Split(New Char() {"&"c})

        Dim year As String = members(2).Replace("[", String.Empty).Replace("]", String.Empty)
        Dim month As String = members(1).Replace("[", String.Empty).Replace("]", String.Empty)
        Return New DateTime(Int32.Parse(year), Int32.Parse(month), 1)

    End Function 'GetDateReportParameter 
#Region "Events"
    Private Sub Main_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Try
            Me.Cursor = Cursors.WaitCursor
            ConfigureOWC()
            LoadMyReports()
        Catch ex As System.Exception
            Util.ShowErrorMessage(ex)
        Finally
            Me.Cursor = Cursors.Default
        End Try
    End Sub 'DataMiner_Load

    Private Sub tabReporter_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles tabReporter.SelectedIndexChanged
        If tabReporter.SelectedTab.Name.ToLower() = "tabchart" Then
            LoadChart()
        End If
    End Sub 'tabReporter_SelectedIndexChanged

    Private Sub cmbReports_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmbReports.SelectedIndexChanged
        LoadReport(CInt(cmbReports.SelectedValue))
        ConfigureMyReports(CInt(cmbReports.SelectedValue))
        tabReporter.SelectedTab = tabPivot
    End Sub 'cmbReports_SelectedIndexChanged
    Private Sub btnSave_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnSave.Click
        UpdateMyReports()
        Me.Dirty = False
    End Sub 'btnSave_Click
    Private Sub pivotTable_DblClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles pivotTable.DblClick
        If Not IsSelectionValid() Then
            Return
        End If
        RunReport("Monthly Order Summary")
    End Sub 'pivotTable_DblClick


    Private Sub pivotTable_SelectionChange(ByVal sender As Object, ByVal e As System.EventArgs) Handles pivotTable.SelectionChange
        Dim selection As String = pivotTable.SelectionType
        Dim aggregates As PivotAggregates = Nothing
        Dim aggregate As PivotAggregate = Nothing

        If selection = "PivotAggregates" Then
            aggregates = CType(pivotTable.Selection, PivotAggregates)
            aggregate = aggregates(0)
            Debug.WriteLine(aggregate.Cell.RowMember.UniqueName)
            Debug.WriteLine(aggregate.Cell.ColumnMember.UniqueName)
        End If
    End Sub 'pivotTable_SelectionChange


    Private Sub pivotTable_BeforeContextMenu(ByVal sender As Object, ByVal e As AxMicrosoft.Office.Interop.Owc11.IPivotControlEvents_BeforeContextMenuEvent) Handles pivotTable.BeforeContextMenu
        Dim newMenu As Object() = Nothing
        If Not IsSelectionValid() Then
            Return
        End If

        Dim menu As Object() = CType(e.menu.Value, Object())

        newMenu = New Object(menu.Length + 2) {}
        menu.CopyTo(newMenu, 0)
        newMenu((newMenu.Length - 2)) = Nothing
        newMenu((newMenu.Length - 1)) = New Object() {"Show Monthly Orders Summary Report", "Monthly Order Summary"}
        e.menu.Value = newMenu
    End Sub 'pivotTable_BeforeContextMenu


    Private Sub pivotTable_CommandExecute(ByVal sender As Object, ByVal e As AxMicrosoft.Office.Interop.Owc11.IPivotControlEvents_CommandExecuteEvent) Handles pivotTable.CommandExecute
        If TypeOf e.command Is String Then
            RunReport(e.command.ToString())
        End If
    End Sub 'pivotTable_CommandExecute
#End Region
End Class
