Imports Microsoft.AnalysisServices.AdomdClient

Module TimeSeries

    Sub Main()
        CreateSessionModel(2003, 3)
    End Sub

    Private Sub CreateSessionModel(ByVal Year As Integer, ByVal numberMonths As Integer)

        Dim query As String
        Dim connection As AdomdConnection = New AdomdConnection("provider=msolap;data source=localhost;initial catalog=Adventure Works DW")
        Dim command As New AdomdCommand()
        Dim adapter As New AdomdDataAdapter()

        Try
            connection.Open()
            ' Drop mining structure if exists from a previous session
            Try
                command.CommandText = "DROP MINING STRUCTURE SessionForecast_Structure"
                command.Connection = connection
                command.ExecuteNonQuery()
            Catch ex As Exception
                ' No big deal if we fail; structure simply doesn't exists
            End Try

            ' Create a session model
            query = "CREATE SESSION MINING MODEL SessionForecast ( " _
                    & "TimeIndex LONG KEY TIME, " _
                    & "ModelRegion TEXT KEY," _
                    & "Amount LONG CONTINUOUS PREDICT" _
                    & ") Using Microsoft_Time_Series"

            command.CommandText = query
            command.ExecuteNonQuery()

            Dim startMonth As Integer = Int32.Parse(Year.ToString() & "01")
            Dim endMonth As Integer = Int32.Parse(Year.ToString() & "12")

            ' Train the model
            query = "INSERT INTO SessionForecast " _
                    & "(TimeIndex, ModelRegion, Amount) " _
                    & "OPENQUERY([Adventure Works DW], " _
                    & "'Select TimeIndex, ModelRegion, Amount " _
                    & "FROM [vTimeSeries] " _
                    & "WHERE (TimeIndex >= " & startMonth & " And TimeIndex <= " & endMonth & ")" _
                    & "ORDER BY TimeIndex')"


            command.CommandText = query
            command.ExecuteNonQuery()

            Dim predictions As New DataSet
            ' Get predicted results
            query = "SELECT ModelRegion, " _
                    & "(SELECT $TIME, [Amount], PredictVariance([Amount]) " _
                    & "FROM PredictTimeSeries([Amount], @NumberMonths)) " _
                    & "FROM [SessionForecast]"

            command.CommandText = query
            command.Parameters.Add("NumberMonths", numberMonths)
            adapter.SelectCommand = command
            ' Fill the dataset
            adapter.Fill(predictions)

            ' Get the cases
            query = "SELECT ModelRegion, TimeIndex, Amount  FROM [SessionForecast].CASES"
            command.CommandText = query
            adapter.SelectCommand = command
            Dim cases As New DataTable
            adapter.Fill(cases)

            ' Merge cases and predictions
            Dim results As DataTable = PopulatePredictedColumns(predictions.Tables(0))
            results.Merge(cases)

        Catch ex As Exception
            Throw ex
        Finally
            ' Close the connection and destroy session
            ' to destroy the session mining model
            connection.Close(True)
        End Try
    End Sub


    Private Function PopulatePredictedColumns(ByVal predictions As DataTable) As DataTable

        Dim results As New DataTable()
        results.Columns.Add("ModelRegion", System.Type.GetType("System.String"))
        results.Columns.Add("TimeIndex", System.Type.GetType("System.Int64"))
        results.Columns.Add("Amount", System.Type.GetType("System.Int64"))

        Dim nestedRows As DataRow()
        Dim relation As DataRelation = predictions.DataSet.Relations(0)

        For Each caseRow As DataRow In predictions.Rows
            nestedRows = caseRow.GetChildRows(relation)
            For Each nestedRow As DataRow In nestedRows
                results.Rows.Add(caseRow(0), nestedRow(0), nestedRow(1))
            Next
        Next

        Return results
    End Function
End Module
