Imports System.Data
Imports System.Data.SqlClient
Imports Microsoft.Reporting.WinForms

Public Class Main
    ''' <summary>
    ''' Generates a local report by binding it to an ADO.NET dataset
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RunLocal()
        Dim salesForecast As New DataSet
        Dim command As New SqlCommand("CreateSessionModel", New SqlConnection(My.Settings.AdventureWorksDWConnectionString))
        Dim adapter As New SqlDataAdapter()
        Dim dataset As New SalesForecast

        Try
            Me.Cursor = Cursors.WaitCursor
            reportViewer.Reset()
            command.CommandTimeout = 60 ' to prevent timeout if the SSAS service is not initialized
            command.CommandType = CommandType.StoredProcedure
            ' The CreateSesssionModel stored procedure takes two parameters
            command.Parameters.Add(New SqlParameter("Year", SqlDbType.Int, 0, ParameterDirection.Input, False, 0, 0, Nothing, DataRowVersion.Default, 2003))
            command.Parameters.Add(New SqlParameter("numberMonths", SqlDbType.Int, 0, ParameterDirection.Input, False, 0, 0, Nothing, DataRowVersion.Default, 3))
            adapter.SelectCommand = command
            adapter.Fill(dataset)

            ' Local report
            reportViewer.ProcessingMode = ProcessingMode.Local
            reportViewer.LocalReport.EnableHyperlinks = True
            'Set RDL file
            reportViewer.LocalReport.ReportPath = "Sales Forecast.rdlc"
            ' Supply a DataTable for the report data source
            reportViewer.LocalReport.DataSources.Add(New ReportDataSource("SalesForecast_Table", dataset.Tables(0)))
            ' Set the report parameters
            Dim parameters(1) As ReportParameter
            parameters(0) = New ReportParameter("Year", 2003)
            parameters(1) = New ReportParameter("NumberMonths", 3)
            reportViewer.LocalReport.SetParameters(parameters)
            reportViewer.RefreshReport()

        Catch ex As Exception
            Throw ex
        Finally
            Me.Cursor = Cursors.Default
        End Try
    End Sub
    ''' <summary>
    ''' Generates a remote report that runs under the context of the Report Server
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RunRemote()

        Try
            Me.Cursor = Cursors.WaitCursor
            ' Remote report
            reportViewer.ProcessingMode = Microsoft.Reporting.WinForms.ProcessingMode.Remote
            reportViewer.ServerReport.ReportServerUrl = New Uri(My.Settings.ReportServerEndPoint)
            reportViewer.ServerReport.ReportPath = "/SOS OLAP/Sales Forecast"

            ' Set the report parameters
            Dim parameters(1) As ReportParameter
            parameters(0) = New ReportParameter("Year", 2003)
            parameters(1) = New ReportParameter("NumberMonths", 3)
            reportViewer.ServerReport.SetParameters(parameters)
            ' The report is always processed asynchronously
            ' Use the RenderingComplete event if you want to be notified 
            ' when the report is ready
            reportViewer.RefreshReport()

        Catch ex As Exception
            Throw ex
        Finally
            Me.Cursor = Cursors.Default
        End Try
    End Sub
    Private Sub btnRun_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRun.Click
        If radRemote.Checked Then
            RunRemote()
        Else
            RunLocal()
        End If
    End Sub

    ''' <summary>
    ''' A drillthrough navigation action is attached to the Amount textbox
    ''' in the Sales Forecast report (both local and remote)
    ''' Since we don't have a local Daily Product Sales report,
    ''' we will ingore the drillthrough action
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub reportViewer_Drillthrough(ByVal sender As System.Object, ByVal e As Microsoft.Reporting.WinForms.DrillthroughEventArgs) Handles reportViewer.Drillthrough

        If (TypeOf e.Report Is LocalReport) Then
            ' cancel the event for local reports because 
            ' the Daily Product Sales is not available as a local report
            MsgBox("Drillthrough is not implemented when the " & e.ReportPath & " is requested locally.")
            e.Cancel = True
        End If
    End Sub

    ''' <summary>
    ''' Demonstrates a hyperlink action which is attached to the    
    ''' ModelRegion textbox in the local version of the SalesForecast report only (Sales Forecast.rdlc)
    ''' as the following expression ="region:" & Fields!ModelRegion.Value
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub reportViewer_Hyperlink(ByVal sender As System.Object, ByVal e As Microsoft.Reporting.WinForms.HyperlinkEventArgs) Handles reportViewer.Hyperlink

        Dim uri As New Uri(e.Hyperlink)

        ' Are we targeting the region?
        If uri.Scheme = "region" Then
            e.Cancel = True

            ' Update the region details
            Dim regionEditor As New RegionEditor()
            ' Remove teh escape characters
            regionEditor.SelectedRegion = System.Web.HttpUtility.UrlDecode(uri.AbsolutePath)

            ' Simulate updating data
            If regionEditor.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
                Me.Refresh()
                ' The user has updated the report data; refresh the report
                RunLocal()
            End If
        End If
    End Sub
End Class
