Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Data.SqlClient
Imports Microsoft.AnalysisServices

Module Main

    Sub Main()
        CreatePartition("SOS OLAP", "Internet Sales", 3)
    End Sub 'CreatePartitions

    ''' <summary>
    '''  Creates a new partition that contains data for the specified number of months
    ''' </summary>
    ''' <param name="cubeName">The Cube Name</param>
    ''' <param name="measureGroupName">The Measure Group where the partition will be created</param>
    ''' <param name="months">The number of months for the partition data</param>
    ''' <remarks></remarks>
    Public Sub CreatePartition(ByVal cubeName As String, ByVal measureGroupName As String, ByVal months As Integer)

        Dim part As Partition
        Dim mg As MeasureGroup
        Dim maxOrderDateKey As Integer = 0
        Dim svr As New Server()

        svr.Connect("Data Source=(local)")

        ' Get the Adventure Works cube
        Dim db As Database = svr.Databases.GetByName("SOS OLAP")
        Dim cube As Cube = db.Cubes.GetByName(cubeName)

        mg = cube.MeasureGroups.GetByName(measureGroupName)

        ' Get the last partition
        part = mg.Partitions(mg.Partitions.Count - 1)

        If TypeOf part.Source Is QueryBinding = False Then
            Throw New ApplicationException("The partition must have Query Binding")
        End If

        ' Get the last date index from the partition query
        Dim lastDateIndex = GetLastDateIndex(part)

        ' Get the new date index.
        Dim newDateIndex As Integer = GetNewDateIndex(lastDateIndex, months)

        ' If there are no enough data in the partition, exit
        If newDateIndex = 0 Then
            Return
        End If

        ' Get the fact table name from the old partition
        Dim factTableName As String = GetFactTableName(CType(part.Source, QueryBinding).QueryDefinition.ToLower())
        ' Create a new partition
        part = mg.Partitions.Add(measureGroupName & " _" & newDateIndex)
        part.StorageMode = StorageMode.Molap
        ' Create the query text
        Dim queryText As String = String.Format("SELECT * FROM {0} WHERE OrderDateKey >= {1} and OrderDateKey <= {2}", factTableName, lastDateIndex + 1, newDateIndex)
        part.Source = New QueryBinding(db.DataSources(0).ID, queryText)
        ' Save the changes
        part.Update()
        ' Process the partition
        part.Process()

        svr.Disconnect()

    End Sub
    ''' <summary>
    ''' Return the last date key for which data is found in the last partition
    ''' </summary>
    ''' <param name="part">The last partition</param>
    ''' <returns>The last date index assuming it is integer</returns>
    ''' <remarks></remarks>
    Private Function GetLastDateIndex(ByVal part As Partition) As Integer
        ' Get the query definition
        Dim query As String = CType(part.Source, QueryBinding).QueryDefinition
        ' Search for "<=" to get to the last date index, e.g. WHERE OrderDateKey >= 185 AND OrderDateKey <= 549
        Dim pos As Integer = query.IndexOf("<=") + 2
        Dim dateIndex As String = query.Substring(pos).Trim()

        Return Int32.Parse(dateIndex)
    End Function
    ''' <summary>
    ''' Returns the last date index for the new partition scope
    ''' </summary>
    ''' <param name="dateIndex">The last date index</param>
    ''' <param name="months">Month offset</param>
    ''' <returns>The new date index or 0 if no data. </returns>
    ''' <remarks></remarks>
    Private Function GetNewDateIndex(ByVal dateIndex As Integer, ByVal months As Integer) As Integer

        ' Open a connection to the data warehouse
        Dim conn As New SqlConnection("data source=localhost;initial catalog=AdventureWorksDW;Integrated Security=SSPI")

        conn.Open()

        ' Create a command
        Dim cmd As New SqlCommand()
        cmd.Connection = conn

        ' Retrieve the date corresponding to the key 
        cmd.CommandText = "select FullDateAlternateKey from DimTime WHERE TimeKey = " & dateIndex

        Dim tempDate As Date = CDate(cmd.ExecuteScalar())

        ' Add the required number of months 
        tempDate = tempDate.AddMonths(months)

        ' Is the old partition still current?
        If (DateTime.Now < tempDate) Then
            Return 0
        End If

        ' Get the end date index from the Time dimension table
        cmd.CommandText = "select TimeKey from DimTime WHERE FullDateAlternateKey = '" & tempDate.ToString("d") & "'"

        Dim lastIndex As Object = cmd.ExecuteScalar()

        If lastIndex Is Nothing Then
            Return 0
        Else
            Return CInt(lastIndex)
        End If


    End Function
    ''' <summary>
    ''' Returns the fact table name from the partition query
    ''' </summary>
    ''' <param name="query">The partition query</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetFactTableName(ByVal query As String)
        Dim pos As Integer = query.IndexOf("from ") + 5
        Dim endPos As Integer = query.IndexOf("where")
        Return query.Substring(pos, endPos - pos).Trim()
    End Function

End Module
